/*
 *
 *    This file is part of NSA ( NullSearchAccess ).
 *
 *  NSA ( NullSearchAccess ) is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  NSA ( NullSearchAccess ) is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with NSA ( NullSearchAccess ).  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "global.h"
#include "typedef.h"

int
test_imap_access (void)
{
  int ret;
  struct socket_s *socket = NULL;

  socket = create_socket();
  if (!socket)
    return -1;

  ret = connect_socket(socket);
  if (ret != -1)
    {
      ret = identify_imap(socket->socket);
      if (!ret)
        logout_suession_imap(socket->socket);
      if (ret)
        ret = 2;
    }

  free_socket(socket);
  return ret;
}

int
identify_imap (int socket)
{
  char imap[500] = {0};
  size_t size;
  int ret;

  size = sizeof(imap)-1;
  ret = xrecv(imap, size, socket);
  if (ret <= -1)
    return ERR_SOCKET;

  ret = check_answer_proto(imap, OK_ACCEUIL_IMAP);
  if (ret == -1)
    return ERR_ACC;

  sprintf(imap, "%s LOGIN %s %s\r\n", CMD_AUTH_IMAP, p_opt->login, p_opt->passwd);
  ret = xsend(imap, strlen(imap), socket);
  if (ret == -1)
    return ERR_SOCKET;
 
  ret = xrecv(imap, size, socket);
  if (ret < 1)
    return ERR_SOCKET;

  ret = check_answer_proto(imap, SUCCESS_AUTH_IMAP);
  if (ret == -1)
    return ERR_PASSWD;

  return ret;
}

void
logout_suession_imap (int socket)
{
  char quit[50] = {0};
  int ret;

  ret = xsend(QUIT_IMAP_STR, strlen(QUIT_IMAP_STR), socket);
  if (ret == -1)
    return;

  ret = xrecv(quit, sizeof(quit)-1, socket);
  if (ret > 0)
    ret = check_answer_proto(quit, QUIT_ANSWER_IMAP);

  if (ret < 1 && debug)
    fprintf(stderr, "logout session imap failed\n");
}

