/*
 *
 *    This file is part of NSA ( NullSearchAccess ).
 *
 *  NSA ( NullSearchAccess ) is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  NSA ( NullSearchAccess ) is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with NSA ( NullSearchAccess ).  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <sys/ipc.h>
#include <sys/shm.h>
#include <stdio.h>

#include "global.h"
#include "typedef.h"

key_t
create_key (void)
{
  key_t key;

  key = ftok(program_name, 1);
  if (key == -1)
    xexit(ERR_CREATE_KEY);

  return key;
}

int
create_shared_memory (key_t key, int size)
{
  int id;

  id = shmget(key, size, IPC_CREAT | 0666);
  if (id == -1)
    xexit(ERR_CREATE_SHARED_MEM);

  return id;
}

char *
attach_shared_memory (int id)
{
  char *memory = NULL;

  memory = shmat(id, NULL, 0);
  if (memory == (char*)-1 && debug)
    {
      fprintf(stderr, "[shmat()] attached memory failed.\n");
      return NULL;
    }

  return memory;
}

void
detach_shared_memory (char *memory)
{
  int ret;

  ret = shmdt(memory);
  if (ret == -1)
    xexit(ERR_DETACH_MEM);
}
     
void
shared_memory_destroy (int id)
{
  int ret;

  ret = shmctl(id, IPC_RMID, 0);
  if (ret == -1)
    xexit(ERR_DESTOY_SHARED_MEM);
}
