/*
 *
 *    This file is part of NSA ( NullSearchAccess ).
 *
 *  NSA ( NullSearchAccess ) is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  NSA ( NullSearchAccess ) is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with NSA ( NullSearchAccess ).  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <string.h>
#include <stdio.h>

#include "typedef.h"
#include "global.h"

int
test_pop3_access (void)
{
  int ret;
  struct socket_s *socket = NULL;

  socket = create_socket();
  if (!socket)
    return -1;

  ret = connect_socket(socket);
  if (ret != -1 )
    {
      ret = identify_pop3(socket->socket);
      if (ret != -1)
        logout_suession_pop3(socket->socket);
      if (ret)
        ret = 2;
    }

  free_socket(socket);
  return ret;
}

int
identify_pop3 (int socket)
{
  char pop[500] = {0};
  size_t size;
  int ret;

  size = sizeof(pop)-1;
  ret = xrecv(pop, size, socket);
  if (ret <= 0)
    return -1;

  ret = check_answer_proto(pop, CMD_SUCCESS_POP3);
  if (ret == -1)
    return ERR_ACC;

  memset(pop, 0, ret);
  snprintf(pop, size, "USER %s\r\n", p_opt->login);
  ret = xsend(pop, strlen(pop), socket);
  if (ret == -1)
    return ERR_SOCKET;

  ret = xrecv(pop, size, socket);
  if (ret <= 0)
    return ERR_SOCKET;

  ret = check_answer_proto(pop, CMD_SUCCESS_POP3);
  if (ret == -1)
    return ERR_LOGIN;

  memset(pop, 0, ret);
  snprintf(pop, size, "PASS %s\r\n", p_opt->passwd);
  ret = xsend(pop, strlen(pop), socket);
  if (ret == -1)
    return ERR_SOCKET;

  ret = xrecv(pop, size, socket);
  if (ret <= 0)
    return ERR_SOCKET;

  ret = check_answer_proto(pop, CMD_SUCCESS_POP3);
  if (ret == -1)
    return ERR_PASSWD;

  return 0;
}

void
logout_suession_pop3 (int socket)
{
  int ret;
  char quit[50];

  ret = xsend(QUIT_POP3, strlen(QUIT_POP3), socket);
  if (ret != -1)
    return;

  ret = xrecv(quit, sizeof(quit)-1, socket);
  if (ret <= 0)
    return;

  ret = check_answer_proto(quit, CMD_SUCCESS_POP3);
  if (ret == -1 && debug)
    fprintf(stderr, "quit server pop3 failed\n");
}
